@@ -3,9 +3,14 @@ use eva_common::actions::{self, ACTION_TOPIC};
 use eva_common::events::RAW_STATE_TOPIC;
 use eva_common::prelude::*;
 use serde::Deserialize;
-use std::time::Duration;
+use std::time::{Duration, Instant};
 use uuid::Uuid;
 
+#[path = "actt.rs"]
+pub mod actt;
+#[path = "transform.rs"]
+pub mod transform;
+
 pub const ERR_NO_PARAMS: &str = "action params not specified";
 
 #[inline]
@@ -25,7 +30,63 @@ pub enum ItemProp {
     Value,
 }
 
-/// Controller action payload
+/// Controller operation
+pub struct Op {
+    t: Instant,
+    timeout: Duration,
+}
+
+impl Op {
+    #[inline]
+    pub fn new(timeout: Duration) -> Self {
+        Self {
+            t: Instant::now(),
+            timeout,
+        }
+    }
+    #[inline]
+    pub fn for_action(action: &Action) -> Self {
+        Self {
+            t: action.received,
+            timeout: action.timeout,
+        }
+    }
+    pub fn is_timed_out(&self) -> bool {
+        let el = self.t.elapsed();
+        el > self.timeout
+    }
+    pub fn timeout(&self) -> EResult<Duration> {
+        let el = self.t.elapsed();
+        if el > self.timeout {
+            Err(Error::timeout())
+        } else {
+            Ok(self.timeout - el)
+        }
+    }
+    #[inline]
+    pub fn is_enough(&self, expected: Duration) -> bool {
+        self.t.elapsed() + expected < self.timeout
+    }
+    #[inline]
+    pub fn enough(&self, expected: Duration) -> EResult<()> {
+        if self.is_enough(expected) {
+            Ok(())
+        } else {
+            Err(Error::timeout())
+        }
+    }
+    #[inline]
+    pub fn remaining(&self, timeout: Duration) -> EResult<Duration> {
+        let el = self.t.elapsed();
+        if el > timeout {
+            Err(Error::timeout())
+        } else {
+            Ok(timeout - el)
+        }
+    }
+}
+
+/// Controller action object
 #[derive(Deserialize, Debug)]
 #[serde(deny_unknown_fields)]
 pub struct Action {
@@ -36,6 +97,8 @@ pub struct Action {
     priority: u8,
     params: Option<actions::Params>,
     config: Option<Value>,
+    #[serde(skip, default = "Instant::now")]
+    received: Instant,
 }
 
 impl Action {
